@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -45,8 +45,8 @@ FBSurfaceSDL2::FBSurfaceSDL2(FBBackendSDL2& backend,
                              uInt32 width, uInt32 height,
                              ScalingInterpolation inter,
                              const uInt32* staticData)
-  : myBackend(backend),
-    myInterpolationMode(inter)
+  : myBackend{backend},
+    myInterpolationMode{inter}
 {
   createSurface(width, height, staticData);
 }
@@ -104,41 +104,49 @@ const Common::Rect& FBSurfaceSDL2::dstRect() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSDL2::setSrcPos(uInt32 x, uInt32 y)
 {
-  if(x != static_cast<uInt32>(mySrcR.x) || y != static_cast<uInt32>(mySrcR.y))
-  {
-    setSrcPosInternal(x, y);
+  if(setSrcPosInternal(x, y))
     reinitializeBlitter();
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSDL2::setSrcSize(uInt32 w, uInt32 h)
 {
-  if(w != static_cast<uInt32>(mySrcR.w) || h != static_cast<uInt32>(mySrcR.h))
-  {
-    setSrcSizeInternal(w, h);
+  if(setSrcSizeInternal(w, h))
+    reinitializeBlitter();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSDL2::setSrcRect(const Common::Rect& r)
+{
+  const bool posChanged = setSrcPosInternal(r.x(), r.y()),
+             sizeChanged = setSrcSizeInternal(r.w(), r.h());
+
+  if(posChanged || sizeChanged)
     reinitializeBlitter();
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSDL2::setDstPos(uInt32 x, uInt32 y)
 {
-  if(x != static_cast<uInt32>(myDstR.x) || y != static_cast<uInt32>(myDstR.y))
-  {
-    setDstPosInternal(x, y);
+  if(setDstPosInternal(x, y))
     reinitializeBlitter();
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSDL2::setDstSize(uInt32 w, uInt32 h)
 {
-  if(w != static_cast<uInt32>(myDstR.w) || h != static_cast<uInt32>(myDstR.h))
-  {
-    setDstSizeInternal(w, h);
+  if(setDstSizeInternal(w, h))
+    reinitializeBlitter();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSDL2::setDstRect(const Common::Rect& r)
+{
+  const bool posChanged = setDstPosInternal(r.x(), r.y()),
+             sizeChanged = setDstSizeInternal(r.w(), r.h());
+
+  if(posChanged || sizeChanged)
     reinitializeBlitter();
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -176,6 +184,22 @@ void FBSurfaceSDL2::invalidate()
   SDL_FillRect(mySurface, nullptr, 0);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSDL2::invalidateRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h)
+{
+  ASSERT_MAIN_THREAD;
+
+  // Clear the rectangle
+  SDL_Rect tmp;
+  tmp.x = x;
+  tmp.y = y;
+  tmp.w = w;
+  tmp.h = h;
+  // Note: Transparency has to be 0 to clear the rectangle foreground
+  //  without affecting the background display.
+  SDL_FillRect(mySurface, &tmp, 0);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSDL2::free()
 {
