@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -66,7 +66,7 @@ namespace {
 CartridgeCDF::CartridgeCDF(const ByteBuffer& image, size_t size,
                            const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    myImage(make_unique<uInt8[]>(512_KB))
+    myImage{make_unique<uInt8[]>(512_KB)}
 {
   // Copy the ROM image into my buffer
   std::fill_n(myImage.get(), 512_KB, 0);
@@ -107,8 +107,8 @@ CartridgeCDF::CartridgeCDF(const ByteBuffer& image, size_t size,
     reinterpret_cast<uInt16*>(myRAM.data()),
     static_cast<uInt32>(512_KB),
     cBase, cStart, cStack,
-    devSettings ? settings.getBool("dev.thumb.trapfatal") : false, 
-    thumulatorConfiguration(myCDFSubtype), 
+    devSettings ? settings.getBool("dev.thumb.trapfatal") : false,
+    thumulatorConfiguration(myCDFSubtype),
     this);
 
   setInitialState();
@@ -277,7 +277,8 @@ uInt8 CartridgeCDF::peek(uInt16 address)
       if DIGITAL_AUDIO_ON
       {
         // retrieve packed sample (max size is 2K, or 4K of unpacked data)
-        uInt32 sampleaddress = getSample() + (myMusicCounters[0] >> 21);
+
+        uInt32 sampleaddress = getSample() + (myMusicCounters[0] >> (isCDFJplus() ? 13 : 21));
 
         // get sample value from ROM or RAM
         if (sampleaddress < 0x00080000)
@@ -288,7 +289,7 @@ uInt8 CartridgeCDF::peek(uInt16 address)
           peekvalue = 0;
 
         // make sure current volume value is in the lower nybble
-        if ((myMusicCounters[0] & (1<<20)) == 0)
+        if ((myMusicCounters[0] & (1<<(isCDFJplus() ? 12 : 20))) == 0)
           peekvalue >>= 4;
         peekvalue &= 0x0f;
       }
@@ -364,7 +365,7 @@ bool CartridgeCDF::poke(uInt16 address, uInt8 value)
       pointer = getDatastreamPointer(COMMSTREAM);
       if (isCDFJplus()) {
         myDisplayImage[ pointer >> 16 ] = value;
-        pointer += 0x00010000;  // always increment by 1 when writing  
+        pointer += 0x00010000;  // always increment by 1 when writing
       } else {
         myDisplayImage[ pointer >> 20 ] = value;
         pointer += 0x00100000;  // always increment by 1 when writing
@@ -520,6 +521,15 @@ uInt32 CartridgeCDF::thumbCallback(uInt8 function, uInt32 value1, uInt32 value2)
   return 0;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 CartridgeCDF::internalRamGetValue(uInt16 addr) const
+{
+  if(addr < internalRamSize())
+    return myRAM[addr];
+  else
+    return 0;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCDF::save(Serializer& out) const
 {
@@ -697,7 +707,7 @@ uInt8 CartridgeCDF::readFromDatastream(uInt8 index)
     pointer += (increment << 8);
   } else {
     value = myDisplayImage[ pointer >> 20 ];
-    pointer += (increment << 12);   
+    pointer += (increment << 12);
   }
 
   setDatastreamPointer(index, pointer);
