@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,9 +33,8 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Settings::Settings()
+  : myRespository{make_shared<KeyValueRepositoryNoop>()}
 {
-  myRespository = make_shared<KeyValueRepositoryNoop>();
-
   // If no version is recorded with the persisted settings, we set it to zero
   setPermanent(SETTINGS_VERSION_KEY, 0);
   setPermanent("stella.version", "6.2.1");
@@ -136,6 +135,8 @@ Settings::Settings()
 
   // Config files and paths
   setPermanent("romdir", "");
+  setPermanent("userdir", "");
+  setPermanent("saveuserdir", "false");
 
   // ROM browser options
   setPermanent("exitlauncher", "false");
@@ -145,6 +146,7 @@ Settings::Settings()
   setPermanent("launcherres", Common::Size(900, 600));
   setPermanent("launcherfont", "medium");
   setPermanent("launcherroms", "true");
+  setPermanent("launchersubdirs", "false");
   setPermanent("romviewer", "1");
   setPermanent("lastrom", "");
 
@@ -176,6 +178,7 @@ Settings::Settings()
   setPermanent("threads", "false");
   setTemporary("romloadcount", "0");
   setTemporary("maxres", "");
+  setPermanent("initials", "");
   setTemporary("turbo", "0");
 
 #ifdef DEBUGGER_SUPPORT
@@ -251,7 +254,7 @@ void Settings::setRepository(shared_ptr<KeyValueRepository> repository)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Settings::load(const Options& options)
 {
-  Options fromFile =  myRespository->load();
+  Options fromFile = myRespository->load();
   for (const auto& opt: fromFile)
     setValue(opt.first, opt.second, false);
 
@@ -349,11 +352,11 @@ void Settings::validate()
     setValue("cursor", "2");
 
   i = getInt("psense");
-  if(i < 0|| i > Paddles::MAX_ANALOG_SENSE)
+  if(i < Paddles::MIN_ANALOG_SENSE || i > Paddles::MAX_ANALOG_SENSE)
     setValue("psense", "20");
 
   i = getInt("dsense");
-  if(i < 1 || i > 20)
+  if(i < Paddles::MIN_DIGITAL_SENSE || i > Paddles::MAX_DIGITAL_SENSE)
     setValue("dsense", "10");
 
   i = getInt("msense");
@@ -525,14 +528,14 @@ void Settings::usage() const
     << endl
     << "  -saveonexit   <none|current| Automatically save state(s) when exiting\n"
     << "                 all>           emulation\n"
-    << "  -autoslot     <1|0>          Automatically change to next save slot when\n"
+    << "  -autoslot     <0|1>          Automatically change to next save slot when\n"
     << "                                state saving\n"
     << endl
     << "  -rominfo      <rom>          Display detailed information for the given ROM\n"
     << "  -listrominfo                 Display contents of stella.pro, one line per ROM\n"
     << "                                entry\n"
     << endl
-    << "  -exitlauncher <1|0>          On exiting a ROM, go back to the ROM launcher\n"
+    << "  -exitlauncher <0|1>          On exiting a ROM, go back to the ROM launcher\n"
     << "  -launcherpos  <XxY>          Sets the window position in windowed EOM launcher mode\n"
     << "  -launcherdisplay <number>    Sets the display for the ROM launcher\n"
     << "  -launcherres  <WxH>          The resolution to use in ROM launcher mode\n"
@@ -541,10 +544,14 @@ void Settings::usage() const
     << "                 medium|large|\n"
     << "                 large12|large14|\n"
     << "                 large16>\n"
-    << "  -launcherroms <1|0>          Show only ROMs in the launcher (vs. all files)\n"
     << "  -romviewer    <float>        Show ROM info viewer at given zoom level in ROM\n"
     << "                                launcher (use 0 for off)\n"
-    << "  -followlauncher <0|1>        Default ROM path follows launcher navigation\n"
+    << "  -launcherroms    <1|0>       Show only ROMs in the launcher (vs. all files)\n"
+    << "  -launchersubdirs <0|1>       Show files from subdirectories too\n"
+    << "  -romdir          <dir>       Set the path where the ROM launcher will start\n"
+    << "  -followlauncher  <0|1>       Default ROM path follows launcher navigation\n"
+    << "  -userdir         <dir>       Set the path to save user files to\n"
+    << "  -saveuserdir     <0|1>       Update user path when navigating in browser\n"
     << "  -lastrom      <name>         Last played ROM, automatically selected in\n"
     << "                                launcher\n"
     << "  -romloadcount <number>       Number of ROM to load next from multicard\n"
@@ -568,8 +575,6 @@ void Settings::usage() const
     << "  -ctrlrate     <rate>         Rate per second of repeated controller input in\n"
     << "                                UI\n"
     << "  -basic_settings <0|1>        Display only a basic settings dialog\n"
-    << "  -romdir       <dir>          Set the directory where the ROM launcher will\n"
-    << "                                start\n"
     << "  -avoxport     <name>         The name of the serial port where an AtariVox is\n"
     << "                                connected\n"
     << "  -holdreset                   Start the emulator with the Game Reset switch\n"
@@ -712,7 +717,7 @@ void Settings::setValue(const string& key, const Variant& value, bool persist)
 {
   auto it = myPermanentSettings.find(key);
   if(it != myPermanentSettings.end()) {
-    if (persist && it->second != value) myRespository->save(key, value);
+    if (persist && it->second != value && myRespository->atomic()) myRespository->atomic()->save(key, value);
     it->second = value;
   }
   else
@@ -755,5 +760,6 @@ void Settings::migrate()
 {
   while (getInt(SETTINGS_VERSION_KEY) < SETTINGS_VERSION) migrateOne();
 
-  myRespository->save(SETTINGS_VERSION_KEY, SETTINGS_VERSION);
+  if (myRespository->atomic())
+    myRespository->atomic()->save(SETTINGS_VERSION_KEY, SETTINGS_VERSION);
 }
