@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -46,8 +46,8 @@ namespace {
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TIASurface::TIASurface(OSystem& system)
-  : myOSystem(system),
-    myFB(system.frameBuffer())
+  : myOSystem{system},
+    myFB{system.frameBuffer()}
 {
   // Load NTSC filter settings
   myNTSCFilter.loadConfig(myOSystem.settings());
@@ -75,6 +75,17 @@ TIASurface::TIASurface(OSystem& system)
   myBaseTiaSurface = myFB.allocateSurface(TIAConstants::frameBufferWidth*2,
                                           TIAConstants::frameBufferHeight);
 
+  // Create shading surface
+  uInt32 data = 0xff000000;
+
+  myShadeSurface = myFB.allocateSurface(1, 1, ScalingInterpolation::sharp, &data);
+
+  FBSurface::Attributes& attr = myShadeSurface->attributes();
+
+  attr.blending = true;
+  attr.blendalpha = 35; // darken stopped emulation by 35%
+  myShadeSurface->applyAttributes();
+
   myRGBFramebuffer.fill(0);
 
   // Enable/disable threading in the NTSC TV effects renderer
@@ -184,7 +195,7 @@ void TIASurface::setNTSC(NTSCFilter::Preset preset, bool show)
   }
   myOSystem.settings().setValue("tv.filter", int(preset));
 
-  if(show) myFB.showMessage(buf.str());
+  if(show) myFB.showTextMessage(buf.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -221,7 +232,7 @@ void TIASurface::setNTSCAdjustable(int direction)
 
   setNTSC(NTSCFilter::Preset::CUSTOM);
   ntsc().selectAdjustable(direction, text, valueText, value);
-  myOSystem.frameBuffer().showMessage(text, valueText, value);
+  myOSystem.frameBuffer().showGaugeMessage(text, valueText, value);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -232,7 +243,7 @@ void TIASurface::changeNTSCAdjustable(int adjustable, int direction)
 
   setNTSC(NTSCFilter::Preset::CUSTOM);
   ntsc().changeAdjustable(adjustable, direction, text, valueText, newValue);
-  myOSystem.frameBuffer().showMessage(text, valueText, newValue);
+  myOSystem.frameBuffer().showGaugeMessage(text, valueText, newValue);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -243,7 +254,7 @@ void TIASurface::changeCurrentNTSCAdjustable(int direction)
 
   setNTSC(NTSCFilter::Preset::CUSTOM);
   ntsc().changeCurrentAdjustable(direction, text, valueText, newValue);
-  myOSystem.frameBuffer().showMessage(text, valueText, newValue);
+  myOSystem.frameBuffer().showGaugeMessage(text, valueText, newValue);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -259,7 +270,7 @@ void TIASurface::setScanlineIntensity(int direction)
     buf << intensity << "%";
   else
     buf << "Off";
-  myFB.showMessage("Scanline intensity", buf.str(), intensity);
+  myFB.showGaugeMessage("Scanline intensity", buf.str(), intensity);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -361,7 +372,7 @@ inline uInt32 TIASurface::averageBuffers(uInt32 bufOfs)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIASurface::render()
+void TIASurface::render(bool shade)
 {
   uInt32 width = myTIA->width(), height = myTIA->height();
 
@@ -438,6 +449,12 @@ void TIASurface::render()
   if(myScanlinesEnabled)
     mySLineSurface->render();
 
+  if(shade)
+  {
+    myShadeSurface->setDstRect(myTiaSurface->dstRect());
+    myShadeSurface->render();
+  }
+
   if(mySaveSnapFlag)
   {
     mySaveSnapFlag = false;
