@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -26,10 +26,6 @@
 CommandMenu::CommandMenu(OSystem& osystem)
   : DialogContainer(osystem)
 {
-  if (osystem.settings().getBool("minimal_ui"))
-    myBaseDialog = new MinUICommandDialog(myOSystem, *this);
-  else
-    myBaseDialog = new CommandDialog(myOSystem, *this);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -41,5 +37,12 @@ CommandMenu::~CommandMenu()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Dialog* CommandMenu::baseDialog()
 {
+  if (myBaseDialog == nullptr) {
+    if (myOSystem.settings().getBool("minimal_ui"))
+      myBaseDialog = new MinUICommandDialog(myOSystem, *this);
+    else
+      myBaseDialog = new CommandDialog(myOSystem, *this);
+  }
+
   return myBaseDialog;
 }
