@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -40,16 +40,15 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                        EventMode mode)
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss),
-    myEventMode(mode)
+    myEventMode{mode}
 {
-  const int fontHeight   = font.getFontHeight(),
-            lineHeight   = font.getLineHeight(),
-            fontWidth    = font.getMaxCharWidth(),
-            buttonWidth  = font.getStringWidth("Defaults") + fontWidth * 1.25,
-            buttonHeight = font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int VGAP = fontHeight / 4;
+  const int lineHeight   = boss->dialog().lineHeight(),
+            fontWidth    = boss->dialog().fontWidth(),
+            buttonHeight = boss->dialog().buttonHeight(),
+            buttonWidth  = boss->dialog().buttonWidth("Defaults"),
+            VBORDER      = boss->dialog().vBorder(),
+            HBORDER      = boss->dialog().hBorder(),
+            VGAP         = boss->dialog().vGap();
   const int ACTION_LINES = 2;
   int xpos = HBORDER, ypos = VBORDER;
   const int listWidth = _w - buttonWidth - HBORDER * 2 - fontWidth;
@@ -68,6 +67,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
     VarList::push_back(items, "Joystick", Event::Group::Joystick);
     VarList::push_back(items, "Paddles", Event::Group::Paddles);
     VarList::push_back(items, "Keyboard", Event::Group::Keyboard);
+    VarList::push_back(items, "Input Devices & Ports", Event::Group::Devices);
     VarList::push_back(items, "Combo", Event::Group::Combo);
     VarList::push_back(items, "Debug", Event::Group::Debug);
 
@@ -97,6 +97,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos,
                                        buttonWidth, buttonHeight,
                                        "Cancel", kStopMapCmd);
+  myCancelMapButton->setToolTip("Cancel current mapping.");
   myCancelMapButton->setTarget(this);
   myCancelMapButton->clearFlags(Widget::FLAG_ENABLED);
   addFocusWidget(myCancelMapButton);
@@ -106,12 +107,14 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                    buttonWidth, buttonHeight,
                                    "Erase", kEraseCmd);
   myEraseButton->setTarget(this);
+  myEraseButton->setToolTip("Erase any mapping for selected event.");
   addFocusWidget(myEraseButton);
 
   ypos += buttonHeight + VGAP;
   myResetButton = new ButtonWidget(boss, font, xpos, ypos,
                                    buttonWidth, buttonHeight,
                                    "Reset", kResetCmd);
+  myResetButton->setToolTip("Reset mapping for selected event to defaults.");
   myResetButton->setTarget(this);
   addFocusWidget(myResetButton);
 
@@ -132,8 +135,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   xpos = HBORDER;
   ypos = myActionsList->getBottom() + VGAP * 2;
   StaticTextWidget* t;
-  t = new StaticTextWidget(boss, font, xpos, ypos+2, font.getStringWidth("Action"),
-                           fontHeight, "Action", TextAlign::Left);
+  t = new StaticTextWidget(boss, font, xpos, ypos+2, "Action");
 
   myKeyMapping = new EditTextWidget(boss, font, xpos + t->getWidth() + fontWidth, ypos,
                                     _w - xpos - t->getWidth() - fontWidth - HBORDER + 2,
