@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -26,10 +26,15 @@
 #include "FSNode.hxx"
 #include "MD5.hxx"
 #include "OptionsDialog.hxx"
+#include "HighScoresDialog.hxx"
+#include "HighScoresManager.hxx"
 #include "GlobalPropsDialog.hxx"
 #include "StellaSettingsDialog.hxx"
 #include "WhatsNewDialog.hxx"
+#include "ProgressDialog.hxx"
 #include "MessageBox.hxx"
+#include "ToolTip.hxx"
+#include "TimerManager.hxx"
 #include "OSystem.hxx"
 #include "FrameBuffer.hxx"
 #include "FBSurface.hxx"
@@ -56,39 +61,86 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
                                int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h)
+  : Dialog(osystem, parent, osystem.frameBuffer().launcherFont(), "",
+           x, y, w, h)
 {
   myUseMinimalUI = instance().settings().getBool("minimal_ui");
-
-  const GUI::Font& font = instance().frameBuffer().launcherFont();
-  const int fontWidth = font.getMaxCharWidth(),
-            fontHeight = font.getFontHeight(),
-            lineHeight = font.getLineHeight(),
-            HBORDER = fontWidth * 1.25,
-            VBORDER = fontHeight / 2,
-            BUTTON_GAP = fontWidth,
-            LBL_GAP = fontWidth,
-            VGAP = fontHeight / 4,
-            buttonHeight = myUseMinimalUI ? lineHeight - VGAP * 2: lineHeight * 1.25,
+  const int lineHeight   = Dialog::lineHeight(),
+            fontHeight   = Dialog::fontHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            BUTTON_GAP   = Dialog::buttonGap(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
+  const int LBL_GAP      = fontWidth,
+            buttonHeight = myUseMinimalUI ? lineHeight - VGAP * 2: Dialog::buttonHeight(),
             buttonWidth  = (_w - 2 * HBORDER - BUTTON_GAP * (4 - 1));
 
-  int xpos = HBORDER, ypos = VBORDER, lwidth = 0, lwidth2 = 0;
+  int xpos = HBORDER, ypos = VBORDER;
   WidgetArray wid;
-  string lblRom = "Select a ROM from the list" + ELLIPSIS;
+  string lblSelect = "Select a ROM from the list" + ELLIPSIS;
+  string lblAllFiles = "Show all files";
   const string& lblFilter = "Filter";
-  const string& lblAllFiles = "Show all files";
-  const string& lblFound = "XXXX items found";
+  string lblSubDirs = "Incl. subdirectories";
+  string lblFound = "12345 items found";
 
-  lwidth = font.getStringWidth(lblRom);
-  lwidth2 = font.getStringWidth(lblAllFiles) + CheckboxWidget::boxSize(font);
-  int lwidth3 = font.getStringWidth(lblFilter);
-  int lwidth4 = font.getStringWidth(lblFound);
+  tooltip().setFont(_font);
 
-  if(w < HBORDER * 2 + lwidth + lwidth2 + lwidth3 + lwidth4 + fontWidth * 6 + LBL_GAP * 8)
+  int lwSelect = _font.getStringWidth(lblSelect);
+  int cwAllFiles = _font.getStringWidth(lblAllFiles) + CheckboxWidget::prefixSize(_font);
+  int cwSubDirs = _font.getStringWidth(lblSubDirs) + CheckboxWidget::prefixSize(_font);
+  int lwFilter = _font.getStringWidth(lblFilter);
+  int lwFound = _font.getStringWidth(lblFound);
+  int wTotal = HBORDER * 2 + lwSelect + cwAllFiles + cwSubDirs + lwFilter + lwFound
+    + EditTextWidget::calcWidth(_font, "123456") + LBL_GAP * 7;
+  bool noSelect = false;
+
+  if(w < wTotal)
   {
     // make sure there is space for at least 6 characters in the filter field
-    lblRom = "Select a ROM" + ELLIPSIS;
-    lwidth = font.getStringWidth(lblRom);
+    lblSelect = "Select a ROM" + ELLIPSIS;
+    int lwSelectShort = _font.getStringWidth(lblSelect);
+
+    wTotal -= lwSelect - lwSelectShort;
+    lwSelect = lwSelectShort;
+  }
+  if(w < wTotal)
+  {
+    // make sure there is space for at least 6 characters in the filter field
+    lblSubDirs = "Subdir.";
+    int cwSubDirsShort = _font.getStringWidth(lblSubDirs) + CheckboxWidget::prefixSize(_font);
+
+    wTotal -= cwSubDirs - cwSubDirsShort;
+    cwSubDirs = cwSubDirsShort;
+  }
+  if(w < wTotal)
+  {
+    // make sure there is space for at least 6 characters in the filter field
+    lblAllFiles = "All files";
+    int cwAllFilesShort = _font.getStringWidth(lblAllFiles) + CheckboxWidget::prefixSize(_font);
+
+    wTotal -= cwAllFiles - cwAllFilesShort;
+    cwAllFiles = cwAllFilesShort;
+  }
+  if(w < wTotal)
+  {
+    // make sure there is space for at least 6 characters in the filter field
+    lblFound = "12345 found";
+    int lwFoundShort = _font.getStringWidth(lblFound);
+
+    wTotal -= lwFound - lwFoundShort;
+    lwFound = lwFoundShort;
+    myShortCount = true;
+  }
+  if(w < wTotal)
+  {
+    // make sure there is space for at least 6 characters in the filter field
+    lblSelect = "";
+    int lwSelectShort = _font.getStringWidth(lblSelect);
+
+    // wTotal -= lwSelect - lwSelectShort; // dead code
+    lwSelect = lwSelectShort;
+    noSelect = true;
   }
 
   if(myUseMinimalUI)
@@ -99,35 +151,55 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   #if defined(RETRON77)
     ver << " for RetroN 77";
   #endif
-    new StaticTextWidget(this, font, 0, ypos, _w, fontHeight,
+    new StaticTextWidget(this, _font, 0, ypos, _w, fontHeight,
                          ver.str(), TextAlign::Center);
     ypos += lineHeight;
   }
 
   // Show the header
-  new StaticTextWidget(this, font, xpos, ypos, lblRom);
+  new StaticTextWidget(this, _font, xpos, ypos, lblSelect);
   // Shop the files counter
-  xpos = _w - HBORDER - lwidth4;
-  myRomCount = new StaticTextWidget(this, font, xpos, ypos,
-                                    lwidth4, fontHeight,
+  xpos = _w - HBORDER - lwFound;
+  myRomCount = new StaticTextWidget(this, _font, xpos, ypos,
+                                    lwFound, fontHeight,
                                     "", TextAlign::Right);
 
   // Add filter that can narrow the results shown in the listing
   // It has to fit between both labels
   if(!myUseMinimalUI && w >= 640)
   {
-    int fwidth = std::min(15 * fontWidth, xpos - lwidth3 - lwidth2 - lwidth - HBORDER - LBL_GAP * 8);
+    int fwFilter = std::min(EditTextWidget::calcWidth(_font, "123456789012345"),
+                            xpos - cwSubDirs - lwFilter - cwAllFiles
+                            - lwSelect - HBORDER - LBL_GAP * (noSelect ? 5 : 7));
+
     // Show the filter input field
-    xpos -= fwidth + LBL_GAP;
-    myPattern = new EditTextWidget(this, font, xpos, ypos - 2, fwidth, lineHeight, "");
+    xpos -= fwFilter + LBL_GAP;
+    myPattern = new EditTextWidget(this, _font, xpos, ypos - 2, fwFilter, lineHeight, "");
+    myPattern->setToolTip("Enter filter text to reduce file list.\n"
+                          "Use '*' and '?' as wildcards.");
+
     // Show the "Filter" label
-    xpos -= lwidth3 + LBL_GAP;
-    new StaticTextWidget(this, font, xpos, ypos, lblFilter);
+    xpos -= lwFilter + LBL_GAP;
+    new StaticTextWidget(this, _font, xpos, ypos, lblFilter);
+
+    // Show the subdirectories checkbox
+    xpos -= cwSubDirs + LBL_GAP * 2;
+    mySubDirs = new CheckboxWidget(this, _font, xpos, ypos, lblSubDirs, kSubDirsCmd);
+    ostringstream tip;
+    tip << "Search files in subdirectories too.";
+    mySubDirs->setToolTip(tip.str());
+
     // Show the checkbox for all files
-    xpos -= lwidth2 + LBL_GAP * 3;
-    myAllFiles = new CheckboxWidget(this, font, xpos, ypos, lblAllFiles, kAllfilesCmd);
+    if(noSelect)
+      xpos = HBORDER;
+    else
+      xpos -= cwAllFiles + LBL_GAP;
+    myAllFiles = new CheckboxWidget(this, _font, xpos, ypos, lblAllFiles, kAllfilesCmd);
+    myAllFiles->setToolTip("Uncheck to show ROM files only.");
+
     wid.push_back(myAllFiles);
     wid.push_back(myPattern);
+    wid.push_back(mySubDirs);
   }
 
   // Add list with game titles
@@ -138,7 +210,7 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   if(romWidth > 0) romWidth += HBORDER;
   int listWidth = _w - (romWidth > 0 ? romWidth + fontWidth : 0) - HBORDER * 2;
   xpos = HBORDER;  ypos += lineHeight + VGAP;
-  myList = new FileListWidget(this, font, xpos, ypos, listWidth, listHeight);
+  myList = new FileListWidget(this, _font, xpos, ypos, listWidth, listHeight);
   myList->setEditable(false);
   myList->setListMode(FilesystemNode::ListMode::All);
   wid.push_back(myList);
@@ -162,12 +234,12 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
 
   // Add textfield to show current directory
   xpos = HBORDER;
-  ypos += myList->getHeight() + VGAP * 2;
-  lwidth = font.getStringWidth("Path") + LBL_GAP;
-  myDirLabel = new StaticTextWidget(this, font, xpos, ypos+2, lwidth, fontHeight,
+  ypos += myList->getHeight() + VGAP;
+  lwSelect = _font.getStringWidth("Path") + LBL_GAP;
+  myDirLabel = new StaticTextWidget(this, _font, xpos, ypos+2, lwSelect, fontHeight,
                                     "Path", TextAlign::Left);
-  xpos += lwidth;
-  myDir = new EditTextWidget(this, font, xpos, ypos, _w - xpos - HBORDER, lineHeight, "");
+  xpos += lwSelect;
+  myDir = new EditTextWidget(this, _font, xpos, ypos, _w - xpos - HBORDER, lineHeight, "");
   myDir->setEditable(false, true);
   myDir->clearFlags(Widget::FLAG_RETAIN_FOCUS);
 
@@ -176,59 +248,64 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
     // Add four buttons at the bottom
     xpos = HBORDER;  ypos = _h - VBORDER - buttonHeight;
   #ifndef BSPF_MACOS
-    myStartButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 0) / 4, buttonHeight,
+    myStartButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 0) / 4, buttonHeight,
                                      "Select", kLoadROMCmd);
     wid.push_back(myStartButton);
 
     xpos += (buttonWidth + 0) / 4 + BUTTON_GAP;
-    myPrevDirButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 1) / 4, buttonHeight,
+    myPrevDirButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 1) / 4, buttonHeight,
                                        "Go Up", kPrevDirCmd);
     wid.push_back(myPrevDirButton);
 
     xpos += (buttonWidth + 1) / 4 + BUTTON_GAP;
-    myOptionsButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 3) / 4, buttonHeight,
+    myOptionsButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 3) / 4, buttonHeight,
                                        "Options" + ELLIPSIS, kOptionsCmd);
     wid.push_back(myOptionsButton);
 
     xpos += (buttonWidth + 2) / 4 + BUTTON_GAP;
-    myQuitButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 4) / 4, buttonHeight,
+    myQuitButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 4) / 4, buttonHeight,
                                     "Quit", kQuitCmd);
     wid.push_back(myQuitButton);
   #else
-    myQuitButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 0) / 4, buttonHeight,
+    myQuitButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 0) / 4, buttonHeight,
                                     "Quit", kQuitCmd);
     wid.push_back(myQuitButton);
 
     xpos += (buttonWidth + 0) / 4 + BUTTON_GAP;
-    myOptionsButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 1) / 4, buttonHeight,
+    myOptionsButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 1) / 4, buttonHeight,
                                        "Options" + ELLIPSIS, kOptionsCmd);
     wid.push_back(myOptionsButton);
 
     xpos += (buttonWidth + 1) / 4 + BUTTON_GAP;
-    myPrevDirButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 2) / 4, buttonHeight,
+    myPrevDirButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 2) / 4, buttonHeight,
                                        "Go Up", kPrevDirCmd);
     wid.push_back(myPrevDirButton);
 
     xpos += (buttonWidth + 2) / 4 + BUTTON_GAP;
-    myStartButton = new ButtonWidget(this, font, xpos, ypos, (buttonWidth + 3) / 4, buttonHeight,
+    myStartButton = new ButtonWidget(this, _font, xpos, ypos, (buttonWidth + 3) / 4, buttonHeight,
                                      "Select", kLoadROMCmd);
     wid.push_back(myStartButton);
   #endif
+    myStartButton->setToolTip("Start emulation of selected ROM.");
   }
   if(myUseMinimalUI) // Highlight 'Rom Listing'
     mySelectedItem = 0;
   else
-    mySelectedItem = 2;
+    mySelectedItem = 3;
 
   addToFocusList(wid);
 
   // Create (empty) context menu for ROM list options
-  myMenu = make_unique<ContextMenu>(this, osystem.frameBuffer().launcherFont(), EmptyVarList);
+  myMenu = make_unique<ContextMenu>(this, _font, EmptyVarList);
 
   // Create global props dialog, which is used to temporarily override
   // ROM properties
   myGlobalProps = make_unique<GlobalPropsDialog>(this,
-    myUseMinimalUI ? osystem.frameBuffer().launcherFont() : osystem.frameBuffer().font());
+    myUseMinimalUI ? _font : osystem.frameBuffer().font());
+
+  // since we cannot know how many files there are, use are really high value here
+  myList->progress().setRange(0, 50000, 5);
+  myList->progress().setMessage("        Filtering files" + ELLIPSIS + "        ");
 
   // Do we show only ROMs or all files?
   bool onlyROMs = instance().settings().getBool("launcherroms");
@@ -278,6 +355,16 @@ void LauncherDialog::reload()
 {
   myMD5List.clear();
   myList->reload();
+  myPendingReload = false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::tick()
+{
+  if(myPendingReload && myReloadTime < TimerManager::getTicks() / 1000)
+    reload();
+
+  Dialog::tick();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -297,6 +384,10 @@ void LauncherDialog::loadConfig()
     instance().settings().setValue("stella.version", STELLA_VERSION);
   }
 
+  bool subDirs = instance().settings().getBool("launchersubdirs");
+  if (mySubDirs) mySubDirs->setState(subDirs);
+  myList->setIncludeSubDirs(subDirs);
+
   // Assume that if the list is empty, this is the first time that loadConfig()
   // has been called (and we should reload the list)
   if(myList->getList().empty())
@@ -319,6 +410,8 @@ void LauncherDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::saveConfig()
 {
+  if (mySubDirs)
+    instance().settings().setValue("launchersubdirs", mySubDirs->getState());
   if(instance().settings().getBool("followlauncher"))
     instance().settings().setValue("romdir", myList->currentDir().getShortPath());
 }
@@ -335,18 +428,90 @@ void LauncherDialog::updateUI()
 
   // Indicate how many files were found
   ostringstream buf;
-  buf << (myList->getList().size() - 1) << " items found";
+  buf << (myList->getList().size() - 1) << (myShortCount ? " found" : " items found");
   myRomCount->setLabel(buf.str());
 
   // Update ROM info UI item
   loadRomInfo();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+size_t LauncherDialog::matchWithJoker(const string& str, const string& pattern)
+{
+  if(str.length() >= pattern.length())
+  {
+    // optimize a bit
+    if(pattern.find('?') != string::npos)
+    {
+      for(size_t pos = 0; pos < str.length() - pattern.length() + 1; ++pos)
+      {
+        bool found = true;
+
+        for(size_t i = 0; found && i < pattern.length(); ++i)
+          if(pattern[i] != str[pos + i] && pattern[i] != '?')
+            found = false;
+
+        if(found)
+          return pos;
+      }
+    }
+    else
+      return str.find(pattern);
+  }
+  return string::npos;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool LauncherDialog::matchWithWildcards(const string& str, const string& pattern)
+{
+  string pat = pattern;
+
+  // remove leading and trailing '*'
+  size_t i = 0;
+  while(pat[i++] == '*');
+  pat = pat.substr(i - 1);
+
+  i = pat.length();
+  while(pat[--i] == '*');
+  pat.erase(i + 1);
+
+  // Search for first '*'
+  size_t pos = pat.find('*');
+
+  if(pos != string::npos)
+  {
+    // '*' found, split pattern into left and right part, search recursively
+    const string leftPat = pat.substr(0, pos);
+    const string rightPat = pat.substr(pos + 1);
+    size_t posLeft = matchWithJoker(str, leftPat);
+
+    if(posLeft != string::npos)
+      return matchWithWildcards(str.substr(pos + posLeft), rightPat);
+    else
+      return false;
+  }
+  // no further '*' found
+  return matchWithJoker(str, pat) != string::npos;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool LauncherDialog::matchWithWildcardsIgnoreCase(const string& str, const string& pattern)
+{
+  string in = str;
+  string pat = pattern;
+
+  BSPF::toUpperCase(in);
+  BSPF::toUpperCase(pat);
+
+  return matchWithWildcards(in, pat);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::applyFiltering()
 {
   myList->setNameFilter(
     [&](const FilesystemNode& node) {
+      myList->incProgress();
       if(!node.isDirectory())
       {
         // Do we want to show only ROMs or all files?
@@ -355,7 +520,7 @@ void LauncherDialog::applyFiltering()
 
         // Skip over files that don't match the pattern in the 'pattern' textbox
         if(myPattern && myPattern->getText() != "" &&
-          !BSPF::containsIgnoreCase(node.getName(), myPattern->getText()))
+           !matchWithWildcardsIgnoreCase(node.getName(), myPattern->getText()))
           return false;
       }
       return true;
@@ -371,16 +536,15 @@ float LauncherDialog::getRomInfoZoom(int listHeight) const
 
   if(zoom > 0.F)
   {
-    const GUI::Font& font = instance().frameBuffer().launcherFont();
     const GUI::Font& smallFont = instance().frameBuffer().smallFont();
-    const int fontWidth = font.getMaxCharWidth(),
-              HBORDER = fontWidth * 1.25;
+    const int fontWidth = Dialog::fontWidth(),
+              HBORDER   = Dialog::hBorder();
 
     // upper zoom limit - at least 24 launchers chars/line and 7 + 4 ROM info lines
     if((_w - (HBORDER * 2 + fontWidth + 30) - zoom * TIAConstants::viewableWidth)
-       / font.getMaxCharWidth() < MIN_LAUNCHER_CHARS)
+       / fontWidth < MIN_LAUNCHER_CHARS)
     {
-      zoom = float(_w - (HBORDER * 2 + fontWidth + 30) - MIN_LAUNCHER_CHARS * font.getMaxCharWidth())
+      zoom = float(_w - (HBORDER * 2 + fontWidth + 30) - MIN_LAUNCHER_CHARS * fontWidth)
         / TIAConstants::viewableWidth;
     }
     if((listHeight - 12 - zoom * TIAConstants::viewableHeight) <
@@ -419,7 +583,7 @@ void LauncherDialog::setRomInfoFont(const Common::Size& area)
   for(size_t i = 0; i < sizeof(FONTS) / sizeof(FontDesc); ++i)
   {
     // only use fonts <= launcher fonts
-    if(instance().frameBuffer().launcherFont().getFontHeight() >= FONTS[i].height)
+    if(Dialog::fontHeight() >= FONTS[i].height)
     {
       if(area.h >= uInt32(MIN_ROMINFO_ROWS * FONTS[i].height + 2
          + MIN_ROMINFO_LINES * FONTS[i].height)
@@ -455,6 +619,8 @@ void LauncherDialog::handleContextMenu()
     myGlobalProps->open();
   else if(cmd == "reload")
     reload();
+  else if(cmd == "highscores")
+    openHighScores();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -469,9 +635,28 @@ void LauncherDialog::showOnlyROMs(bool state)
 void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
   // Grab the key before passing it to the actual dialog and check for
-  // Control-R (reload ROM listing)
-  if(StellaModTest::isControl(mod) && key == KBDK_R)
-    reload();
+  // context menu keys
+  if(StellaModTest::isControl(mod))
+  {
+    switch(key)
+    {
+      case KBDK_P:
+        myGlobalProps->open();
+        break;
+
+      case KBDK_H:
+        if(instance().highScores().enabled())
+          openHighScores();
+        break;
+
+      case KBDK_R:
+        reload();
+        break;
+
+      default:
+        break;
+    }
+  }
   else
 #if defined(RETRON77)
     // handle keys used by R77
@@ -560,19 +745,22 @@ Event::Type LauncherDialog::getJoyAxisEvent(int stick, JoyAxis axis, JoyDir adir
 void LauncherDialog::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   // Grab right mouse button for context menu, send left to base class
-  if(b == MouseButton::RIGHT)
+  if(b == MouseButton::RIGHT
+     && x + getAbsX() >= myList->getLeft() && x + getAbsX() <= myList->getRight()
+     && y + getAbsY() >= myList->getTop() && y + getAbsY() <= myList->getBottom())
   {
     // Dynamically create context menu for ROM list options
     VariantList items;
 
-    if (!currentNode().isDirectory() && Bankswitch::isValidRomName(currentNode()))
-      VarList::push_back(items, "Power-on options" + ELLIPSIS, "override");
-    VarList::push_back(items, "Reload listing", "reload");
+    if(!currentNode().isDirectory() && Bankswitch::isValidRomName(currentNode()))
+      VarList::push_back(items, " Power-on options" + ELLIPSIS + "   Ctrl+P", "override");
+    if(instance().highScores().enabled())
+      VarList::push_back(items, " High scores" + ELLIPSIS + "        Ctrl+H", "highscores");
+    VarList::push_back(items, " Reload listing      Ctrl+R ", "reload");
     myMenu->addItems(items);
 
     // Add menu at current x,y mouse location
     myMenu->show(x + getAbsX(), y + getAbsY(), surface().dstRect());
-
   }
   else
     Dialog::handleMouseDown(x, y, b, clickCount);
@@ -589,6 +777,11 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       reload();
       break;
 
+    case kSubDirsCmd:
+      myList->setIncludeSubDirs(mySubDirs->getState());
+      reload();
+      break;
+
     case kLoadROMCmd:
     case FileListWidget::ItemActivated:
       saveConfig();
@@ -614,9 +807,23 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case EditableWidget::kChangedCmd:
+    case EditableWidget::kAcceptCmd:
+    {
+      bool subDirs = mySubDirs->getState();
+
+      myList->setIncludeSubDirs(subDirs);
       applyFiltering();  // pattern matching taken care of directly in this method
-      reload();
+
+      if(subDirs && cmd == EditableWidget::kChangedCmd)
+      {
+        // delay (potentially slow) subdirectories reloads until user stops typing
+        myReloadTime = TimerManager::getTicks() / 1000 + myList->getQuickSelectDelay();
+        myPendingReload = true;
+      }
+      else
+        reload();
       break;
+    }
 
     case kQuitCmd:
       saveConfig();
@@ -662,7 +869,7 @@ void LauncherDialog::loadRom()
       instance().settings().setValue("romdir", currentNode().getParent().getShortPath());
   }
   else
-    instance().frameBuffer().showMessage(result, MessagePosition::MiddleCenter, true);
+    instance().frameBuffer().showTextMessage(result, MessagePosition::MiddleCenter, true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -693,10 +900,22 @@ void LauncherDialog::openSettings()
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::openHighScores()
+{
+  // Create an options dialog, similar to the in-game one
+  if(myHighScoresDialog == nullptr)
+    myHighScoresDialog = make_unique<HighScoresDialog>(instance(), parent(), _w, _h,
+                                                       Menu::AppMode::launcher);
+
+  myHighScoresDialog->open();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::openWhatsNew()
 {
   if(myWhatsNewDialog == nullptr)
-    myWhatsNewDialog = make_unique<WhatsNewDialog>(instance(), parent(), _font, _w, _h);
+    myWhatsNewDialog = make_unique<WhatsNewDialog>(instance(), parent(), _w, _h);
   myWhatsNewDialog->open();
+
 }
