@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -37,7 +37,7 @@ RomInfoWidget::RomInfoWidget(GuiObject* boss, const GUI::Font& font,
                              int x, int y, int w, int h,
                              const Common::Size& imgSize)
   : Widget(boss, font, x, y, w, h),
-    myAvail(imgSize)
+    myAvail{imgSize}
 {
   _flags = Widget::FLAG_ENABLED;
   _bgcolor = kDlgColor;
@@ -108,20 +108,13 @@ void RomInfoWidget::parseProperties(const FilesystemNode& node)
       myProperties.get(PropType::Cart_Name) + ".png";
 
   // Read the PNG file
-  try
-  {
-    instance().png().loadImage(filename, *mySurface);
+  mySurfaceIsValid = loadPng(filename);
 
-    // Scale surface to available image area
-    const Common::Rect& src = mySurface->srcRect();
-    float scale = std::min(float(myAvail.w) / src.w(), float(myAvail.h) / src.h()) *
-        instance().frameBuffer().hidpiScaleFactor();
-    mySurface->setDstSize(uInt32(src.w() * scale), uInt32(src.h() * scale));
-    mySurfaceIsValid = true;
-  }
-  catch(const runtime_error& e)
+  // Try to load a default image if not ROM image exists
+  if(!mySurfaceIsValid)
   {
-    mySurfaceErrorMsg = e.what();
+    mySurfaceIsValid = loadPng(instance().snapshotLoadDir().getPath() +
+                               "default_snapshot.png");
   }
 #else
   mySurfaceErrorMsg = "PNG image loading not supported";
@@ -172,17 +165,41 @@ void RomInfoWidget::parseProperties(const FilesystemNode& node)
     myRomInfo.push_back("Controllers: " + (left + " (left), " + right + " (right)"));
   if (bsDetected != "")
     myRomInfo.push_back("Type: " + Bankswitch::typeToDesc(Bankswitch::nameToType(bsDetected)));
+
+  setDirty();
 }
 
+#ifdef PNG_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool RomInfoWidget::loadPng(const string& filename)
+{
+  try
+  {
+    instance().png().loadImage(filename, *mySurface);
+
+    // Scale surface to available image area
+    const Common::Rect& src = mySurface->srcRect();
+    float scale = std::min(float(myAvail.w) / src.w(), float(myAvail.h) / src.h()) *
+      instance().frameBuffer().hidpiScaleFactor();
+    mySurface->setDstSize(uInt32(src.w() * scale), uInt32(src.h() * scale));
+
+    return true;
+  }
+  catch(const runtime_error& e)
+  {
+    mySurfaceErrorMsg = e.what();
+  }
+  return false;
+}
+#endif
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomInfoWidget::drawWidget(bool hilite)
 {
   FBSurface& s = dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
-
   const int yoff = myAvail.h + 10;
 
-  s.fillRect(_x+2, _y+2, _w-4, _h-4, onTop ? _bgcolor : _bgcolorlo);
+  s.fillRect(_x+2, _y+2, _w-4, _h-4, _bgcolor);
   s.frameRect(_x, _y, _w, _h, kColor);
   s.frameRect(_x, _y+yoff, _w, _h-yoff, kColor);
 
@@ -204,7 +221,7 @@ void RomInfoWidget::drawWidget(bool hilite)
   {
     uInt32 x = _x + ((_w - _font.getStringWidth(mySurfaceErrorMsg)) >> 1);
     uInt32 y = _y + ((yoff - _font.getLineHeight()) >> 1);
-    s.drawString(_font, mySurfaceErrorMsg, x, y, _w - 10, onTop ? _textcolor : _shadowcolor);
+    s.drawString(_font, mySurfaceErrorMsg, x, y, _w - 10, _textcolor);
   }
 
   int xpos = _x + 8, ypos = _y + yoff + 5;
@@ -224,7 +241,8 @@ void RomInfoWidget::drawWidget(bool hilite)
         break;
     }
     int lines = s.drawString(_font, info, xpos, ypos, _w - 16, _font.getFontHeight() * 3,
-                             onTop ? _textcolor : _shadowcolor);
+                             _textcolor);
     ypos += _font.getLineHeight() + (lines - 1) * _font.getFontHeight();
   }
+  clearDirty();
 }
