@@ -8,13 +8,15 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "OSystem.hxx"
+#include "FrameBuffer.hxx"
 #include "Version.hxx"
 
 #include "WhatsNewDialog.hxx"
@@ -22,20 +24,21 @@
 constexpr int MAX_CHARS = 64; // maximum number of chars per line
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-WhatsNewDialog::WhatsNewDialog(OSystem& osystem, DialogContainer& parent, const GUI::Font& font,
+WhatsNewDialog::WhatsNewDialog(OSystem& osystem, DialogContainer& parent,
                                int max_w, int max_h)
 #if defined(RETRON77)
-  : Dialog(osystem, parent, font, "What's New in Stella " + string(STELLA_VERSION) + " for RetroN 77?")
+  : Dialog(osystem, parent, osystem.frameBuffer().font(),
+           "What's New in Stella " + string(STELLA_VERSION) + " for RetroN 77?")
 #else
-  : Dialog(osystem, parent, font, "What's New in Stella " + string(STELLA_VERSION) + "?")
+  : Dialog(osystem, parent, osystem.frameBuffer().font(),
+           "What's New in Stella " + string(STELLA_VERSION) + "?")
 #endif
 {
-  const int fontHeight = _font.getFontHeight(),
-    fontWidth = _font.getMaxCharWidth(),
-    buttonHeight = _font.getLineHeight() * 1.25;
-  const int VGAP = fontHeight / 4;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int ypos = _th + VBORDER;
 
   // Set preliminary dimensions
@@ -43,18 +46,18 @@ WhatsNewDialog::WhatsNewDialog(OSystem& osystem, DialogContainer& parent, const
           max_w, max_h);
 
 #if defined(RETRON77)
-  add(ypos, "fixed CDF cartridges crash");
-  add(ypos, "fixed crash with SaveKey ROMs (EEPROM file issues)");
-  add(ypos, "fixed bug with launcher not remembering last selected ROM");
+  add(ypos, "increased sample size for CDFJ+");
+  add(ypos, "fixed navigation bug in Video & Audio settings dialog");
+  add(ypos, "fixed autofire bug for trackball controllers");
 #else
-  add(ypos, "added basic text cut/copy/paste from/to UI");
-  add(ypos, "added color parameters to 'Custom' palette");
-  add(ypos, "improved AtariVox-USB adaptor autodetection");
-  add(ypos, "fixed fullscreen mode, aspect correction and pixel-exact snapshots");
-  add(ypos, "fixed reduced ARM emulation performance for CDF ROMs");
-  add(ypos, "fixed crash with SaveKey ROMs (EEPROM file issues)");
-  add(ypos, "fixed Atari mouse autodetection");
-  add(ypos, "fixed bug with launcher not remembering last selected ROM");
+  add(ypos, "added high scores saving");
+  add(ypos, "enhanced cut/copy/paste and undo/redo for text editing");
+  add(ypos, "added mouse support for text editing");
+  add(ypos, "added wildcard support to launcher dialog filter");
+  add(ypos, "added option to search subdirectories in launcher");
+  add(ypos, "added tooltips to many UI items");
+  add(ypos, "added sound to Time Machine playback");
+  add(ypos, "moved settings, properties etc. to an SQLite database");
   add(ypos, ELLIPSIS + " (for a complete list see 'docs/Changes.txt')");
 #endif
 
@@ -75,10 +78,9 @@ WhatsNewDialog::WhatsNewDialog(OSystem& osystem, DialogContainer& parent, const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void WhatsNewDialog::add(int& ypos, const string& text)
 {
-  const int lineHeight = _font.getLineHeight(),
-    fontHeight = _font.getFontHeight(),
-    fontWidth = _font.getMaxCharWidth(),
-    HBORDER = fontWidth * 1.25;
+  const int lineHeight = Dialog::lineHeight(),
+            fontHeight = Dialog::fontHeight(),
+            HBORDER    = Dialog::hBorder();
   const string DOT = "\x1f";
   string txt = DOT + " " + text;
 
